package com.loganmagnan.blockhealth;

import com.loganmagnan.blockhealth.managers.BlockManager;
import com.loganmagnan.blockhealth.managers.PlayerDataManager;
import com.loganmagnan.blockhealth.utils.ClassRegistrationUtils;
import com.loganmagnan.blockhealth.utils.Utils;
import com.loganmagnan.blockhealth.utils.command.CommandFramework;
import com.loganmagnan.blockhealth.utils.menusystem.PlayerMenuUtil;
import lombok.Getter;
import com.loganmagnan.blockhealth.utils.config.FileConfig;
import com.loganmagnan.blockhealth.utils.config.file.Config;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.plugin.java.JavaPlugin;

import java.util.HashMap;

@Getter
public class BlockHealth extends JavaPlugin {

    @Getter private static BlockHealth instance;

    private Config mainConfig;
    private FileConfig messagesConfig;

    private PlayerDataManager playerDataManager;
    private BlockManager blockManager;

    private HashMap<Player, PlayerMenuUtil> playerMenuUtilMap = new HashMap<>();

    private CommandFramework commandFramework = new CommandFramework(this);

    @Override
    public void onEnable() {
        instance = this;

        this.saveDefaultConfig();
        this.mainConfig = new Config("config", this);
        this.messagesConfig = new FileConfig(this, "messages.yml");

        Bukkit.getConsoleSender().sendMessage("------------------------------------------------");
        Bukkit.getConsoleSender().sendMessage(Utils.translate("&bBlockHealth &8- &av" + this.getDescription().getVersion()));
        Bukkit.getConsoleSender().sendMessage(Utils.translate("&7Made by &eLoganM Development"));
        Bukkit.getConsoleSender().sendMessage("------------------------------------------------");

        this.loadCommands();
        this.loadManagers();
        this.loadListeners();
        this.loadRunnables();

        if (this.getServer().getPluginManager().getPlugin("DecentHolograms") == null) {
            Bukkit.getConsoleSender().sendMessage(Utils.translate("&cDecentHolograms not found"));
            Bukkit.getConsoleSender().sendMessage(Utils.translate("&cBlockHealth has been disabled"));

            this.getServer().getPluginManager().disablePlugin(this);
        } else {
            Bukkit.getConsoleSender().sendMessage(Utils.translate("&aDecentHolograms has been enabled"));
        }
    }

    @Override
    public void onDisable() {
        instance = null;
    }

    private void loadCommands() {
        ClassRegistrationUtils.loadCommands("me.trixkz.blockhealth.commands");
    }

    private void loadManagers() {
        this.playerDataManager = new PlayerDataManager();
        this.blockManager = new BlockManager();
    }

    private void loadListeners() {
        ClassRegistrationUtils.loadListeners("me.trixkz.blockhealth.listeners");
    }

    private void loadRunnables() {

    }
}
