package com.loganmagnan.blockhealth.managers;

import com.loganmagnan.blockhealth.runnables.BlockHealthRunnable;
import lombok.Getter;
import lombok.Setter;
import com.loganmagnan.blockhealth.BlockHealth;
import com.loganmagnan.blockhealth.utils.Utils;
import net.md_5.bungee.api.ChatMessageType;
import net.md_5.bungee.api.chat.TextComponent;
import org.bukkit.Material;
import org.bukkit.block.Block;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ThreadLocalRandom;
import java.util.concurrent.atomic.AtomicInteger;

@Getter
@Setter
public class BlockManager {

    private BlockHealth main = BlockHealth.getInstance();

    private Map<Block, AtomicInteger> blocks = new HashMap<Block, AtomicInteger>();
    private Map<Material, AtomicInteger> blockHealths = new HashMap<Material, AtomicInteger>();
    private Map<Material, AtomicInteger> damageAmountsPerBreak = new HashMap<Material, AtomicInteger>();

    private int defaultBlockHealth;
    private int defaultMinimumDamageAmountPerBreak;
    private int defaultMaximumDamageAmountPerBreak;
    private int defaultDamageAmountPerBreak;

    private BlockHealthRunnable blockHealthRunnable;

    public BlockManager() {
        for (String string : this.main.getMainConfig().getConfig().getConfigurationSection("BLOCKS").getKeys(false)) {
            this.blockHealths.put(Material.valueOf(string.toUpperCase()), new AtomicInteger(this.main.getMainConfig().getConfig().getInt("BLOCKS." + string + ".HEALTH")));
        }

        for (String string : this.main.getMainConfig().getConfig().getConfigurationSection("BLOCKS").getKeys(false)) {
            int minimumDamageAmountPerBreak = this.main.getMainConfig().getConfig().getInt("BLOCKS." + string + ".MINIMUM-DAMAGE-AMOUNT-PER-BREAK");
            int maximumDamageAmountPerBreak = this.main.getMainConfig().getConfig().getInt("BLOCKS." + string + ".MAXIMUM-DAMAGE-AMOUNT-PER-BREAK");
            int damageAmountPerBreak = ThreadLocalRandom.current().nextInt(minimumDamageAmountPerBreak, maximumDamageAmountPerBreak + 1);

            this.damageAmountsPerBreak.put(Material.valueOf(string.toUpperCase()), new AtomicInteger(damageAmountPerBreak));
        }

        this.defaultBlockHealth = this.main.getMainConfig().getConfig().getInt("DEFAULT-BLOCK-HEALTH");
        this.defaultMinimumDamageAmountPerBreak = this.main.getMainConfig().getConfig().getInt("DEFAULT-MINIMUM-DAMAGE-AMOUNT-PER-BREAK");
        this.defaultMaximumDamageAmountPerBreak = this.main.getMainConfig().getConfig().getInt("DEFAULT-MAXIMUM-DAMAGE-AMOUNT-PER-BREAK");
        this.defaultDamageAmountPerBreak = ThreadLocalRandom.current().nextInt(this.defaultMinimumDamageAmountPerBreak, this.defaultMaximumDamageAmountPerBreak + 1);
        this.blockHealthRunnable = new BlockHealthRunnable();
    }

    public int getBlockHealth(Block block) {
        if (!this.isBlockRegistered(block)) {
            int blockHealth = this.getBlockMaximumHealth(block);

            this.addBlock(block, blockHealth);

            return blockHealth;
        }

        return this.blocks.get(block).get();
    }

    public int getBlockMaximumHealth(Block block) {
        Material material = block.getType();

        int blockMaximumHealth = 0;

        if (this.blockHealths.containsKey(material)) {
            blockMaximumHealth = this.blockHealths.get(material).get();
        } else {
            blockMaximumHealth = this.defaultBlockHealth;
        }

        return blockMaximumHealth;
    }

    public int getDamageAmountPerBreak(Block block) {
        Material material = block.getType();

        int damageAmountPerBreak = 0;

        if (this.damageAmountsPerBreak.containsKey(material)) {
            damageAmountPerBreak = this.damageAmountsPerBreak.get(material).get();
        } else {
            damageAmountPerBreak = this.defaultDamageAmountPerBreak;
        }

        return damageAmountPerBreak;
    }

    public void addBlock(Block block, int blockHealth) {
        this.blocks.put(block, new AtomicInteger(blockHealth));
    }

    public void removeBlock(Block block) {
        this.blocks.remove(block);
    }

    public void sendPlayerBlockHealthMessage(Player player, Block block) {
        player.sendMessage(ChatMessageType.ACTION_BAR, TextComponent.fromLegacyText(Utils.translate("&bBlock Health: &f" + this.getBlockHealth(block))));
    }

    public boolean isBlockRegistered(Block block) {
        return this.blocks.containsKey(block);
    }

    public boolean isBlockBroken(Block block) {
        return this.getBlockHealth(block) <= 0;
    }

    public void damageBlock(Player player, Block block) {
        ItemStack itemStack = player.getItemInHand();

        int amount = 0;
        amount = this.getDamageAmountPerBreak(block);

        // to do
        // look for the custom items damage amounts

        int blockHealth = this.getBlockHealth(block);
        blockHealth -= amount;

        this.blocks.get(block).set(blockHealth);

        if (this.isBlockBroken(block)) {
            this.removeBlock(block);

            block.setType(Material.AIR);
        }
    }
}
